<?php
if (!function_exists('_')) {
	function _($s) {
		return $s;
	}
}

class phpUserTrack_Test {

	var $_phpver;

	function phpUserTrack_Test() {

		/* Store the PHP version information. */
		$this->_phpver = $this->splitPHPVersion(phpversion());

		/* We want to be as verbose as possible here. */
		error_reporting(E_ALL);

		/* Set character encoding. */
		header('Content-type: text/html; charset=utf-8');
		header('Vary: Accept-Language');
	}

	function splitPHPVersion($version) {
		/* First pick off major version, and lower-case the rest. */
		if ((strlen($version) >= 3) && ($version[1] == '.')) {
			$phpver['major'] = substr($version, 0, 3);
			$version = substr(strtolower($version), 3);
		} else {
			$phpver['major'] = $version;
			$phpver['class'] = 'unknown';
			return $phpver;
		}

		if ($version[0] == '.') {
			$version = substr($version, 1);
		}

		/* Next, determine if this is 4.0b or 4.0rc; if so, there is no
		   minor, the rest is the subminor, and class is set to beta. */
		$s = strspn($version, '0123456789');
		if ($s == 0) {
			$phpver['subminor'] = $version;
			$phpver['class'] = 'beta';
			return $phpver;
		}

		/* Otherwise, this is non-beta;  the numeric part is the minor,
		   the rest is either a classification (dev, cvs) or a subminor
		   version (rc<x>, pl<x>). */
		$phpver['minor'] = substr($version, 0, $s);
		if ((strlen($version) > $s) && (($version[$s] == '.') || ($version[$s] == '-'))) {
			$s++;
		}
		$phpver['subminor'] = substr($version, $s);
		if (($phpver['subminor'] == 'cvs') || ($phpver['subminor'] == 'dev') || (substr($phpver['subminor'], 0, 2) == 'rc')) {
			unset ($phpver['subminor']);
			$phpver['class'] = 'dev';
		} else {
			if (!$phpver['subminor']) {
				unset ($phpver['subminor']);
			}
			$phpver['class'] = 'release';
		}

		return $phpver;
	}

	function phpModuleCheck($modlist) {
		$output = '';
		$output_array = array ();
		global $passTests;

		foreach ($modlist as $key => $val) {
			$error_msg = $mod_test = $status_out = $fatal = null;
			$entry = array ();

			if (is_array($val)) {
				$descrip = $val['descrip'];
				$fatal = !empty ($val['fatal']);
				if (isset ($val['phpver']) && (version_compare(phpversion(), $val['phpver']) == -1)) {
					$mod_test = true;
					$status_out = 'N/A';
				}
				if (isset ($val['error'])) {
					$error_msg = $val['error'];
				}
			} else {
				$descrip = $val;
			}

			if (is_null($status_out)) {
				$mod_test = extension_loaded($key);
				$status_out = $this->_status($mod_test, $fatal);
			}

			$entry[] = $descrip;
			$entry[] = $status_out;

			if (!is_null($error_msg) && !$mod_test) {
				$entry[] = $error_msg;
				if (!$fatal) {
					$entry[] = 1;
				}
			}

			$output .= $this->_outputLine($entry);

			if ($fatal && !$mod_test) {
				$passTests = false;
			}
		}

		return $output;
	}
	
	function PEARModuleCheck($pear_list)
    {
        $output = '';

        /* Turn tracking of errors on. */
        ini_set('track_errors', 1);

        /* Print the include_path. */
        $output .= $this->_outputLine(array("<strong>PEAR Search Path (PHP's include_path)</strong>", '&nbsp;<tt>' . ini_get('include_path') . '</tt>'));

        /* Go through module list. */
        $succeeded = array();
        foreach ($pear_list as $key => $val) {
            $entry = array();

            /* If this module depends on another module that we
             * haven't succesfully found, fail the test. */
            if (!empty($val['depends']) && empty($succeeded[$val['depends']])) {
                $result = false;
            } else {
                $result = @include_once $val['path'];
            }
            $error_msg = $val['error'];
            if ($result && isset($val['function'])) {
                $func_output = call_user_func($val['function']);
                if ($func_output) {
                    $result = false;
                    $error_msg = $func_output;
                }
            }
            $entry[] = $key;
            $entry[] = $this->_status($result, !empty($val['required']));

            if ($result) {
                $succeeded[$key] = true;
            } else {
                if (!empty($val['required'])) {
                    $error_msg .= ' THIS IS A REQUIRED MODULE!';
                }
                $entry[] = $error_msg;
                if (empty($val['required'])) {
                    $entry[] = 1;
                }
            }

            $output .= $this->_outputLine($entry);
        }

        /* Restore previous value of 'track_errors'. */
        ini_restore('track_errors');

        return $output;
    }

	function requiredFileCheck($file_list) {
		$output = '';
		global $passTests;

		foreach ($file_list as $key => $val) {
			$entry = array ();
			$result = file_exists('./' . $key);

			$entry[] = $key;
			$entry[] = $this->_status($result);

			if (!$result) {
				if (empty ($val)) {
					$entry[] = 'The file <code>' . $key . '</code> appears to be missing. You probably just forgot to copy <code>' . $key . '.dist</code> over. While you do that, take a look at the settings and make sure they are appropriate for your site.';
				} else {
					$entry[] = $val;
				}
				$passTests = false;
			}

			$output .= $this->_outputLine($entry);
		}

		return $output;
	}

	function configFilesMissing($app, $appBase, $files, $additional = array ()) {
		/* Try to load a basic framework if we're testing an app other than
		 * the Horde base files. */
		if ($app != 'Horde') {
			$GLOBALS['registry'] = & Registry :: singleton();
			$GLOBALS['registry']->pushApp('phpUserTrack', false);
		}

		if (!is_array($files)) {
			$files = array (
				$files
			);
		}
		$files = array_merge($files, array_keys($additional));

		/* Try to auto-create missing .dist files. */
		$indices = array_keys($files);
		foreach ($indices as $index) {
			if (is_readable($appBase . '/config/' . $files[$index])) {
				unset ($files[$index]);
			} else {
				if (file_exists($appBase . '/config/' . $files[$index] . '.dist') && @ copy($appBase . '/config/' . $files[$index] . '.dist', $appBase . '/config/' . $files[$index])) {
					unset ($files[$index]);
				}
			}
		}

		/* Return if we have no missing files left. */
		if (!count($files)) {
			return;
		}

	}

	function getPhpVersionInformation() {
		$output = & new stdClass;
		$url = urlencode($_SERVER['PHP_SELF']);
		$vers_check = true;

		$testscript = '.' . '/test.php';
		$output->phpinfo = $testscript . '?mode=phpinfo&url=' . $url;
		$output->extensions = $testscript . '?mode=extensions&url=' . $url;
		$output->version = phpversion();
		$output->major = $this->_phpver['major'];
		if (isset ($this->_phpver['minor'])) {
			$output->minor = $this->_phpver['minor'];
		}
		if (isset ($this->_phpver['subminor'])) {
			$output->subminor = $this->_phpver['subminor'];
		}
		$output->class = $this->_phpver['class'];

		$output->status_color = 'red';
		if ($output->major < '5.0') {
			$output->status = 'This version of PHP is not supported. You need to upgrade to at least PHP 5.0';
			$vers_check = false;
		}
		elseif (($output->major == '5.0') || ($output->major == '5.1')) {
			$output->status = 'You are running a supported version of PHP.';
			$output->status_color = 'green';
		} else {
			$output->status = 'Wow, a mystical version of PHP from the future. Let <a href="mailto:mini@chatetheory.com">mini@chatetheory.com</a> know what version you have so we can fix this script.';
			$output->status_color = 'orange';
		}

		if (!$vers_check) {
			$output->version_check = 'Horde requires PHP 4.3.0 or greater.';
		}

		return $output;
	}

	function _status($bool, $required = true) {
		if ($bool) {
			return '<span class="supported">Yes</span>';
		}
		elseif ($required) {
			global $passTests;
			$passTests = false;
			return '<span class="notsupported">No</span>';
		} else {
			return '<span class="notrequired">No</span>';
		}
	}

	function _outputLine($entry) {
		$output = '<li>' . array_shift($entry) . ': ' . array_shift($entry);
		if (!empty ($entry)) {
			$msg = array_shift($entry);
			$output .= '<br /><span class="' . (empty ($entry) || !array_shift($entry) ? 'notsupported' : 'notrequired') . '">' . $msg . '</span>' . "\n";
		}
		$output .= '</li>' . "\n";

		return $output;
	}

}