<?php
/**
 * IP address (unique user) abstraction class
 * @author Christopher Troup <mini@chatetheory.com>
 * @package phpUserTrack
 * @subpackage Libraries
 */
 
 require_once 'libdb.php';
 
class phpusertrack_ip extends phpusertrack_db {
	/**
	 * Profile index
	 * @var integer
	 */
	var $_profileId = null;
	
	/**
	 * IP Address
	 * @var string
	 */
	var $_ip = null;
	
	/**
	 * Session identification string
	 * @var string
	 */
	var $_sessId = null;

	/**
	 * Class contructor
	 * @param string $ip The IP address of requesting user
	 * @return boolean Success of object creation
	 */
	function phpusertrack_ip($ip = null) {
		if ($ip != null) {
			$this->_ip = $ip; 
			$this->_sessId = session_id();
			$this->_profileId = $this->lookupIdFromSession();
		} else {
			return false;
		}
		return true;
	}

	/**
	 * Find that users profile index from their session identification string
	 * @return integer Profile index
	 */
	function lookupIdFromSession() {
		global $conf;
		$result = $this->_query('select id from ' . $conf['ipTable'] . ' where session_id="' . $this->_sessId . '"');
		if (!empty($result)) {
			return $result[0]['id'];
		} else {
			return null;
		}
	}
	
	/**
	 * Return the profile index number
	 * @return integer Profile index number
	 */
	function getProfileId() {
		return $this->_profileId;
	}
	
	/**
	 * Return IP address
	 * @return string IP address
	 */
	function getIp() {
		return $this->_ip;
	}
	
	/** 
	 * Return session identification string
	 * @return string Session identification string
	 */
	function getSessId() {
		return $this->_sessId;
	}
	
	/**
	 * Set classes IP address
	 * @param string $ip IP Address to set the class to
	 */
	function setIp($ip) {
		$this->_ip = $ip;
		$this->_profileId = $this->lookupIdFromSession();
	}
	
	/**
	 * Insert new profile into the database
	 * @param string $browser Browser name user is using
	 * @param string $browser_verion Browser verison user is using
	 * @param string $platform Operating System
	 */
	function insertProfile($browser, $browser_version, $platform) {
		global $conf;
		$this->_insert('insert into ' . $conf['ipTable'] . 
			' set session_id="' . $this->_sessId . 
			'", ip="' . $this->_ip . '", ' . 
			'browser="' . $browser . 
			'", browser_version="' . $browser_version . '", ' .
			'platform="' . $platform . '"');
		$this->_profileId = $this->lookupIdFromSession();
	}

}
?>
